#include <avr/io.h>
#include <stdint.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "nokia5110.h"
#include "DS3231.h"
#include "simple_twi.h"
#include "dht.h"
#include "notes.h"
#include "pwmmusic.h"
#include <math.h>
#define DHT11_PIN 3
#define CRT_YEAR 2018
#define DEFAULT_VOLUME 200

volatile int set_hour;
volatile int set_min;
volatile int set_month;
volatile int set_day;
volatile int alarm = 0;
volatile int is_confirmed = 0;
volatile int alarm_tone = 0;

/* returneaza ultima zi din luna primita ca parametru */
int get_last_day(int month, int year)
{
	if (month == 1 || month == 3 || month == 5 || month == 7
	||month == 8 || month == 10 || month == 12)
	return 31;
	if (month == 2) {
		//an bisect
		if ((year % 4 == 0 && year % 100 == 0) || year % 400 == 0)
		return 29;
		else
		return 28;
	}
	return 30;
}

void InitMusic()
{
	// configurare OC1B ca iesire

	DDRD |= (1 << DDD4);

	// configurare timer pentru PWM
	TCCR1A |= (1 << COM1B1);

	// TOP = ICR1, PS = 8
	TCCR1B |= (1 << WGM13) | (1 << CS11);
}

/* amanare alarma cu 2 minute */
void snooze_alarm(int nr) {
	int crt_hour = set_hour;
	if (set_min + nr >= 60)
		set_hour = (set_hour + 1) % 24;
	set_min = (set_min + nr) % 60;
	
	if (set_hour == 0 && crt_hour != 0)
	set_day = set_day + 1;
	if(set_day > get_last_day(set_month, CRT_YEAR)) {
		set_day = 1;
		set_month = (set_month + 1) % 12;
	}
}

/* pornire alarma */
void PlayMusic( const int* pMusicNotes, uint8_t tempo)
{
	int duration;
	int note;
	int i;
	uint16_t delay = tempo * 1000;
	char data[5];

	while( *pMusicNotes )
	{
		PORTC |= (1 << PC7);
		note = *pMusicNotes;
		pMusicNotes++;

		duration = *pMusicNotes;
		pMusicNotes++;

		if( p == note )
		{
			//pauza
			OCR1B = 0;
		}
		else
		{
			//genereaza sunet
			OCR1B = DEFAULT_VOLUME;

			//setare frecventa
			ICR1H = (note >> 8);
			ICR1L = note;
		}

		//wait duration
		for(i = 0; i < 32 - duration; i++)
		{
			_delay_loop_2( delay );
		}
		
		
		PORTC &= ~(1 << PC7);
		
		nokia_lcd_clear();
		nokia_lcd_set_cursor(0, 15);
		nokia_lcd_write_string("Wake Up!", 2);
		nokia_lcd_render();
		
		/* oprire alarma */
		if ((PINB & (1 << PB3)) == 0) {
			alarm = 0;
			nokia_lcd_clear();
			nokia_lcd_set_cursor(0, 15);
			nokia_lcd_write_string("Dismiss", 2);
			nokia_lcd_render();
			_delay_ms(200);
			nokia_lcd_clear();
			break;		
		}
		
		/* snooze */
		if ((PINB & (1 << PB0)) == 0) {
			OCR1B = 0;
			nokia_lcd_clear();
			nokia_lcd_set_cursor(0, 15);
			nokia_lcd_write_string("Snooze", 2);
			nokia_lcd_render();
			_delay_ms(200);
			nokia_lcd_clear();
			
			int nr = 0;
			while (!((PINB & (1 << PB3)) == 0)) {
				itoa(nr, data, 10);
				nokia_lcd_set_cursor(0, 15);
				nokia_lcd_write_string(data, 2);
				nokia_lcd_write_string("  min", 2);
				nokia_lcd_render();
				_delay_ms(150);
				nokia_lcd_clear();
				
				if ((PINB & (1 << PB1)) == 0) {
					nr--;
					if (nr < 0)
						nr = 0;
				}
				
				if ((PINB & (1 << PB2)) == 0)
					nr++;
			}
			
			snooze_alarm(nr);
			break;
		}
		
	}
	
	//oprire sunet alarma
	OCR1B = 0;
}

void print_month(int month, uint8_t size)
{
	switch(month) {
		case 1:
		nokia_lcd_write_string("JAN", size);
		break;
		case 2:
		nokia_lcd_write_string("FEB", size);
		break;
		case 3:
		nokia_lcd_write_string("MAR", size);
		break;
		case 4:
		nokia_lcd_write_string("APR", size);
		break;
		case 5:
		nokia_lcd_write_string("MAY", size);
		break;
		case 6:
		nokia_lcd_write_string("JUN", size);
		break;
		case 7:
		nokia_lcd_write_string("JUL", size);
		break;
		case 8:
		nokia_lcd_write_string("AUG", size);
		break;
		case 9:
		nokia_lcd_write_string("SEP", size);
		break;
		case 10:
		nokia_lcd_write_string("OCT", size);
		break;
		case 11:
		nokia_lcd_write_string("NOV", size);
		break;
		case 12:
		nokia_lcd_write_string("DEC", size);
		break;
		default:
		break;
	}
}

/* Intrerupere pe butonul PB0 pentru setare alarma */
ISR(PCINT1_vect)
{
	char data[5];
	if ((PINB & (1 << PB0)) == 0 && alarm == 0)
	{
		int nrhours = 0;
		int nrmin = 0;
		alarm = 1;
		is_confirmed = 0;
		
		uint8_t nrs, nrm, nrh, day1, day2, month, year;
		DS3231_getTime(&nrs, &nrm, &nrh, &day1, &day2, &month, &year);
		uint8_t month_no = month;
		int day_no = day2;
		
		//cat timp nu s-a apasat confirmare data
		while(!((PINB & (1 << PB3)) == 0)) {
			nokia_lcd_clear();
			nokia_lcd_set_cursor(0, 15);
			
			if ((PINB & (1 << PB2)) == 0) //setare luna
			{
				month_no++;
				if (month_no > 12)
					month_no = 1;
				_delay_ms(150);
			}
			
			if ((PINB & (1 << PB1)) == 0) //setare zi
			{
				day_no++;
				int last_day = get_last_day(month_no, CRT_YEAR);
				if (day_no > last_day)
					day_no = 1;
				_delay_ms(150);
			}
			
			
			print_month(month_no, 2);
			nokia_lcd_write_string("  ", 2);
			itoa(day_no, data, 10);
			nokia_lcd_write_string(data, 2);
			
			nokia_lcd_render();
		}
		nokia_lcd_clear();
		nokia_lcd_set_cursor(0, 15);
		nokia_lcd_write_string("The date is", 1);
		nokia_lcd_set_cursor(36, 25);
		nokia_lcd_write_string("set", 1);
		nokia_lcd_render();
		_delay_ms(1000);
		nokia_lcd_clear();
				
		//cat timp nu s-a apasat confirmare alarma
		while(!((PINB & (1 << PB3)) == 0) && is_confirmed == 0) {
			nokia_lcd_clear();
			nokia_lcd_set_cursor(0, 15);
			if ((PINB & (1 << PB1)) == 0) //setare ora
			{
				nrhours++;
				if (nrhours == 24)
					nrhours = 0;
				_delay_ms(150);
			}
			
			if ((PINB & (1 << PB2)) == 0) //setare minute
			{
				nrmin++;
				if(nrmin == 60)
					nrmin = 0;
				_delay_ms(150);
			}
			
			if(nrhours < 10)
				nokia_lcd_write_string("0", 2);
			itoa(nrhours, data, 10);
			nokia_lcd_write_string(data, 2);
			nokia_lcd_write_string(" : ", 2);
			if(nrmin < 10)
				nokia_lcd_write_string("0", 2);
			itoa(nrmin, data, 10);
			nokia_lcd_write_string(data, 2);
			nokia_lcd_render();
		}
		
		//s-a apasat confirmare alarma
		nokia_lcd_clear();
		nokia_lcd_set_cursor(0, 15);
		nokia_lcd_write_string("The alarm is", 1);
		nokia_lcd_set_cursor(36, 25);
		nokia_lcd_write_string("set", 1);
		nokia_lcd_render();
		_delay_ms(1000);
		nokia_lcd_clear();
		
		nokia_lcd_set_cursor(0, 15);
		nokia_lcd_write_string("Set the alarm tone:", 1);
		nokia_lcd_render();
		_delay_ms(1000);
		
		//cat timp nu s-a apasat confirmare ton alarma
		while(!((PINB & (1 << PB3)) == 0) && is_confirmed == 0) {
			nokia_lcd_clear();
			nokia_lcd_set_cursor(0, 15);
			if ((PINB & (1 << PB1)) == 0)
			{
				alarm_tone = (alarm_tone + 1) % 4;
				_delay_ms(200);
			}
			
			if ((PINB & (1 << PB2)) == 0)
			{
				alarm_tone = alarm_tone - 1;
				if (alarm_tone == -1)
					alarm_tone = 3;
				_delay_ms(200);
			}
			
			if (alarm_tone == 0) {
				nokia_lcd_write_string("Default", 1);
			} else if (alarm_tone == 1) {
				nokia_lcd_write_string("Fur Elise", 1);
			} else if (alarm_tone == 2) {
				nokia_lcd_write_string("Star Wars", 1);
			} else {
				nokia_lcd_write_string("Hey Jude", 1);
			}
			nokia_lcd_render();
		}
		
		nokia_lcd_clear();
		nokia_lcd_set_cursor(0, 15);
		nokia_lcd_write_string("The alarm tone", 1);
		nokia_lcd_set_cursor(32, 25);
		nokia_lcd_write_string("is set", 1);
		nokia_lcd_render();
		_delay_ms(1000);
		nokia_lcd_clear();
		
		set_month = month_no;
		set_day = day_no;
		set_hour = nrhours;
		set_min = nrmin;
		is_confirmed = 1;
	}
}

/* Intrerupere pe butonul PD6 pentru setare ora si data curenta */
ISR(PCINT3_vect)
{
	char data[5];
	uint8_t nrs, nrm, nrh, day1, day2, month, year;
	DS3231_getTime(&nrs, &nrm, &nrh, &day1, &day2, &month, &year);
	
	if ((PIND & (1 << PD6)) == 0)
	{
		int nrhours = 0;
		int nrmin = 0;
		uint8_t new_month = 1;
		int new_day = 1;
		
		//cat timp nu s-a apasat confirmare data
		while(!((PINB & (1 << PB3)) == 0)) {
			nokia_lcd_clear();
			nokia_lcd_set_cursor(0, 15);
			
			if ((PINB & (1 << PB2)) == 0) //setare luna
			{
				new_month++;
				if (new_month > 12)
				new_month = 1;
				_delay_ms(150);
			}
				
			if ((PINB & (1 << PB1)) == 0) //setare zi
			{
				new_day++;
				int last_day = get_last_day(new_month, year);
				if (new_day > last_day)
					new_day = 1;
				_delay_ms(150);
			}
			
			
			print_month(new_month, 2);
			nokia_lcd_write_string("  ", 2);
			itoa(new_day, data, 10);
			nokia_lcd_write_string(data, 2);
			
			nokia_lcd_render();
		}

		nokia_lcd_clear();
		nokia_lcd_set_cursor(0, 15);
		nokia_lcd_write_string("The date is", 1);
		nokia_lcd_set_cursor(36, 25);
		nokia_lcd_write_string("set", 1);
		nokia_lcd_render();
		_delay_ms(1000);
		nokia_lcd_clear();
		
		//cat timp nu s-a apasat confirmare ora setata
		while(!((PINB & (1 << PB3)) == 0)) {
			nokia_lcd_clear();
			nokia_lcd_set_cursor(0, 15);
			if ((PINB & (1 << PB1)) == 0) //setare ora
			{
				nrhours++;
				if (nrhours == 24)
					nrhours = 0;
				_delay_ms(150);
			}
			
			if ((PINB & (1 << PB2)) == 0) //setare minute
			{
				nrmin++;
				if(nrmin == 60)
					nrmin = 0;
				_delay_ms(150);
			}
			
			if(nrhours < 10)
				nokia_lcd_write_string("0", 2);
			itoa(nrhours, data, 10);
			nokia_lcd_write_string(data, 2);
			nokia_lcd_write_string(" : ", 2);
			if(nrmin < 10)
				nokia_lcd_write_string("0", 2);
			itoa(nrmin, data, 10);
			nokia_lcd_write_string(data, 2);
			nokia_lcd_render();
		}
		
		//s-a apasat confirmare ora
		DS3231_setTime(0, nrmin, nrhours, day1, new_day, new_month, year);
		
		nokia_lcd_clear();
		nokia_lcd_set_cursor(0, 15);
		nokia_lcd_write_string("The clock is", 1);
		nokia_lcd_set_cursor(36, 25);
		nokia_lcd_write_string("set", 1);
		nokia_lcd_render();
		_delay_ms(500);
		nokia_lcd_clear();
	}
}


/* initializeaza intrarile si iesirile */
void IO_init()
{
	
	// butoanele sunt puse ca intrare, cu rezistenta de pull-up
	//buton setare alarma
	DDRB &= ~(1 << PB0);
	PORTB |= (1 << PB0);

	//buton setare ora pentru alarma
	DDRB &= ~(1 << PB1);
	PORTB |= (1 << PB1);
	
	//buton setare minute pentru alarma
	DDRB &= ~(1 << PB2);
	PORTB |= (1 << PB2);
	
	//buton oprire alarma
	DDRB &= ~(1 << PB3);
	PORTB |= (1 << PB3);
	
	//buton setare ora
	DDRD &= ~(1 << PD6);
	PORTD |= (1 << PD6);
	
	//led-ul PD7 este pus ca iesire
	DDRD |= (1 << PD7);
	PORTD &= ~(1 << PD7);

	// Activez PIN CHANGE INTERRUPT ENABLE pentru PCINT15:8 si 31:24
	PCICR |= (1 << PCIE1) | (1 << PCIE3);
	// Activez intreruperea pe butoanele PB0 si PD6
	PCMSK1 |= (1 << PCINT8);
	PCMSK3 |= (1 << PCINT30);
}


/* ora curenta este cea setata in alarma */
void alarm_ready(void) {

	/* Setez pinul PD4 ca iesire. */
	DDRD |= (1 << PD4);

	/* Opresc speaker-ul. */
	PORTD &= ~(1 << PD4);
	
	/* activez led-ul de light al display-ului */
	DDRC |= (1 << PC7);
	PORTC &= ~(1 << PC7);
	
	if (alarm_tone == 1)
		PlayMusic(furelise, 100);
	else
		if (alarm_tone == 2)
			PlayMusic(starwars, 20);
		else
			if (alarm_tone == 3)
				PlayMusic(jude, 35);
			else
				PlayMusic(octave, 40);
	
	/* sting led-ul display-ului */
	PORTC |= (1 << PC7);
	
}

int main(void)
{
	char data[5];
	uint8_t nrs,min,hours,day1,day2,month,year;
	
	int8_t temperature = 0;
	int8_t hummidity = 0;

	char temp_msg[30];
	char humm_msg[30];
	
	twi_INIT();
	nokia_lcd_init();
	nokia_lcd_clear();
	InitMusic();

	/* activare intreruperi */
	sei();
	IO_init();
	_delay_ms(500);
	
	while(1)
	{
		
		nokia_lcd_clear();
		DS3231_getTime(&nrs, &min, &hours, &day1, &day2, &month, &year);
		
		if (min == set_min && hours == set_hour
			&& month == set_month && day2 == set_day && alarm == 1) {
			alarm_ready();
			nokia_lcd_clear();
		}
		
		/* afisare ora */
		if (hours <= 9)
			nokia_lcd_write_string("0", 2);
		itoa(hours,data,10);
		nokia_lcd_write_string(data, 2);
		nokia_lcd_write_string(":", 2);
		if (min <= 9)
			nokia_lcd_write_string("0", 2);
		itoa(min,data,10);
		nokia_lcd_write_string(data, 2);
		
		/* ledul user va sta o secunda aprins si o secunda stins */
		if (nrs % 2 == 0)
			PORTD |= (1 << PD7);
		else
			PORTD &= ~(1 << PD7);
		
		/* afisare zi curenta */
		nokia_lcd_set_cursor(0, 20);
		if (day2 <= 9)
		nokia_lcd_write_string("0", 1);
		itoa(day2,data,10);
		nokia_lcd_write_string(data, 1);
		nokia_lcd_write_string("/", 1);
		print_month(month, 1);
		nokia_lcd_write_string("/", 1);
		itoa(year,data,10);
		nokia_lcd_write_string(data, 1);
		
		/* preluare date de la senzor */
		dht_gettemperaturehumidity(&temperature, &hummidity);
		_delay_ms(100);
		
		/* afisare temperatura si umiditate */
		memset(temp_msg, 0, sizeof(temp_msg));
		sprintf(temp_msg, "Temp: %d C", (int)temperature);
		
		memset(humm_msg, 0, sizeof(humm_msg));
		sprintf(humm_msg, "H: %d %%", (int)hummidity);
		
		nokia_lcd_set_cursor(0, 30);
		nokia_lcd_write_string(temp_msg, 1);
		
		nokia_lcd_set_cursor(0, 40);
		nokia_lcd_write_string(humm_msg, 1);

		nokia_lcd_render();
		
		_delay_ms(500);
		
	}
}
