#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define TWI_BUS_PORT    PORTC
#define TWI_BUS_DDR     DDRC
#define SDA             PC1
#define SCL             PC0

#define MAX_RESTARTS    20
#define I2C_FREQ        400000UL

#define TW_START					0x08
#define TW_REP_START				0x10
// Master Transmitter
#define TW_MT_SLA_ACK				0x18
#define TW_MT_SLA_NACK				0x20
#define TW_MT_DATA_ACK				0x28
#define TW_MT_DATA_NACK				0x30
#define TW_MT_ARB_LOST				0x38
// Master Receiver
#define TW_MR_ARB_LOST				0x38
#define TW_MR_SLA_ACK				0x40
#define TW_MR_SLA_NACK				0x48
#define TW_MR_DATA_ACK				0x50
#define TW_MR_DATA_NACK				0x58
#define TW_BUS_ERROR				0x00
#define TWSR_STATUS_MASK            0xF8
#define TW_STATUS                   TWSR & TWSR_STATUS_MASK
//read or write modes
#define TW_READ                     1
#define TW_WRITE                    0


void twi_INIT(void);



unsigned char twi_START(void);

void twi_STOP(void);

unsigned char twi_writeChar(unsigned char data);


unsigned char twi_readChar_ACK(void);


unsigned char twi_readChar_NACK(void);


unsigned char twi_read (unsigned char slvadrs,   // device address to read from
                        unsigned char numberofbytes,       // number of bytes to read
                        unsigned char adrs_to_read_Within_device,      // device register to start reading from
                        unsigned char *buf);




unsigned char twi_write(unsigned char slvadrs,                      // dslave address of the device to write to
                        unsigned char numberofbytes,                // number of bytes to write
                        unsigned char adrs_to_write_Within_device,  // device register to start writing to
                        unsigned char *buf);


